
//
//  DepthFogging.hs
//
//  Created by Aaron Bossert. (aaronb@z2.com)
//  Copyright (c) 2013 Z2Live. All rights reserved.
//

#if !defined(__fog__)
#define __fog__ 1

uniform Float_h fogStart;
uniform Float_h fogTransition;
uniform Float_h fogFalloff;
uniform Float_h fogDensity;
uniform Vec3_m  fogColor;

varying Vec4_l pixelFog;

// Fog functions defined this way so game specific fog function signatures can be used with the
// our engine materials, despite local attribs being defined after this function
#define GeneratePixelFog() pixelFog = __GeneratePixelFog(pixelPosition.w)
Vec4_l __GeneratePixelFog(Float_h zdepth)
{
    Float_h fog = clamp((zdepth - fogStart) / fogTransition, 0.0, 1.0);

    return Vec4(fogColor, pow(fog, fogFalloff) * fogDensity);
}

#define ApplyPixelFog() outputColor.rgb = __ApplyPixelFog(outputColor.rgb)
Vec3_l __ApplyPixelFog(Vec3_l surfaceRGB)
{
    return mix(surfaceRGB, pixelFog.rgb, pixelFog.a);
}

#endif //__fog__