
//
//  DirectionalLighting.hs
//
//  Created by Joseph Danaher. (Joseph@z2.com)
//  Copyright (c) 2014 Z2Live. All rights reserved.
//

Vec4_h GenerateDirectionalLightColor(
										Vec3_h normal,
										Vec3_h lightDir,


										Vec3_h ambientRGB,
										Float_l ambientLightIntensity,


										Vec3_h lightRGB,
										Float_l diffuseLightIntensity
										)
{

    // compute per-vertex lighting - this is OK because it's a directional light with no specular content
    // Joseph: Adjusted half lambert values for more dramatic lighting. Previously: [ * 0.5 + 0.5 ]
    Float_m lambertianTerm  = max(0.0, dot(normal, lightDir) * 0.8 + 0.2 );
    Vec3_m  lambertianVec   = Vec3(lambertianTerm, lambertianTerm, lambertianTerm);

    // Calculate the directional light and shading
    Vec3_m directionalLight = lambertianVec * (lightRGB * diffuseLightIntensity);

    // Calculate the ambient light
    Vec3_m ambientLight = ambientRGB * ambientLightIntensity;


    return Vec4(ambientLight + directionalLight, 1.0);

}
Vec4_h GenerateDirectionalLightColor_reduct(
                                        Vec3_h normal,
                                        Vec3_h lightDir,


                                        Vec3_h ambientRGB,
                                        Float_l ambientLightIntensity,


                                        Vec3_h lightRGB,
                                        Float_l reductionFactor
                                        )
{

    // compute per-vertex lighting - this is OK because it's a directional light with no specular content
    // Joseph: Adjusted half lambert values for more dramatic lighting. Previously: [ * 0.5 + 0.5 ]
    Float_m lambertianTerm  = max(0.0, dot(normal, lightDir) * 0.8 + 0.2 );
    Vec3_m  lambertianVec   = Vec3(lambertianTerm, lambertianTerm, lambertianTerm);

    // Calculate the directional light and shading
    Vec3_m directionalLight = lambertianVec * lightRGB;

    // Calculate the ambient light
    Vec3_m ambientLight = ambientRGB * ambientLightIntensity;

    return mix( Vec4(1.0), Vec4(ambientLight + directionalLight, 1.0), reductionFactor);

}


Vec4_h GenerateDirectionalLightColor_old(Vec3_h normal, Vec3_h lightDir, Vec3_h ambientRGB, Vec3_h lightRGB)
{

    // compute per-vertex lighting - this is OK because it's a directional light with no specular content
    // Joseph: Adjusted half lambert values for more dramatic lighting. Previously: [ * 0.5 + 0.5 ]
    Float_m lambertianTerm  = max(0.0, dot(normal, lightDir) * 0.8 + 0.2 );
    Vec3_m  lambertianVec   = Vec3(lambertianTerm, lambertianTerm, lambertianTerm);
    return Vec4(ambientRGB + lambertianVec * lightRGB, 1.0);

}

