
//
//  GerstnerWaves.hs
//
//  Created by Aaron Bossert. (aaronb@z2.com)
//

struct WaveData
{
	Float_h	time;
	Float_h	waterLevel;
	Vec4_h frequency;
	Vec4_h phase;
	Vec4_h amplitude;
	Vec4_h dThetaDx;
	Vec4_h dThetaDz;
	Vec4_h waveVectors[4];
	Float_h	attenuation;
};

///	Vertex
///
///	resulting Gerstner wave vertex
struct Vertex
{
	Vec3_h position;
	Vec3_h tangent;
	Vec3_h normal;
	Vec3_h bitangent;
};

///	computeGerstner()
///
///	compute a displaced vertex given a position in the xz plane
///	using the Gerstner wave approach
Vertex computeGerstner(Vec3_h position, in WaveData waveData)
{
	Vec4_h theta = Vec4_h
	(
		dot(waveData.waveVectors[0].xy, position.xz) - waveData.frequency.x * waveData.time + waveData.phase.x,
		dot(waveData.waveVectors[1].xy, position.xz) - waveData.frequency.y * waveData.time + waveData.phase.y,
		dot(waveData.waveVectors[2].xy, position.xz) - waveData.frequency.z * waveData.time + waveData.phase.z,
		dot(waveData.waveVectors[3].xy, position.xz) - waveData.frequency.w * waveData.time + waveData.phase.w
	);

	Vec4_h sinTheta = sin(theta);
	Vec4_h cosTheta = cos(theta);

	Vertex vertex;

	vertex.position.xz = position.xz - waveData.attenuation *
	(
		waveData.waveVectors[0].zw * sinTheta.xx +
		waveData.waveVectors[1].zw * sinTheta.yy +
		waveData.waveVectors[2].zw * sinTheta.zz +
		waveData.waveVectors[3].zw * sinTheta.ww
	);
	vertex.position.y = position.y + dot(waveData.amplitude, vec4(1.0)) + waveData.attenuation * dot(waveData.amplitude, cosTheta);

	vertex.normal = normalize(vec3(dot(waveData.dThetaDx * waveData.attenuation, sinTheta), 1.0, dot(waveData.dThetaDz * waveData.attenuation, sinTheta)));
	vertex.bitangent = cross(vec3(1.0, 0.0, 0.0), vertex.normal);
	vertex.tangent = cross(vertex.normal, vertex.bitangent);

	return vertex;
 }
