
//
//  SpecularHilighting.hs
//
//  Created by Aaron Bossert. (aaronb@z2.com)
//  Copyright (c) 2013 Z2Live. All rights reserved.
//

// Shared by vertex and fragment shaders
Vec3_m GenerateSpecularHighlight(Vec3_h viewReflection, Vec3_h worldNormal, Vec3_h viewDir, Vec3_h lightDir, Vec3_h ambientRGB, Vec3_h lightRGB)
{
	// TODOMBEACH: make these uniforms
	Float_h specularExp = 60.0; // specular shininess
	Float_h fresnelBias =  0.2; // [0.0:1.0] => base (minimum) ambient reflectivity
	Float_h fresnelExp  =  4.0; // ambient reflection (rim-light) falloff

	// reflected ambient
	Float_m facingAmmount   = 1.0 - max(dot(-viewDir, worldNormal), 0.0);
	Float_m fresnel         = fresnelBias + (1.0 - fresnelBias) * pow(facingAmmount, fresnelExp);
	Vec3_m  reflectionColor = ambientRGB * fresnel;

	Float_m phongSpec       = pow(max(0.0, dot(viewReflection, lightDir)), specularExp);
	Vec3_m  specLighting    = reflectionColor + phongSpec * lightRGB;

	return specLighting;
}
