
//
//  abstractDirectionalDiffuse.hs
//
//  Created by Aaron Bossert. (aaronb@z2.com)
//

// This is not built into prometheus because it adds a constant proportion of diffuse light to ambient light
// When prometheusDirectionalDiffuse is added artists should be adjusting ambient and diffuse completely separately
varying Vec4_m directionalLight;

#ifdef Vertex

	#include <prometheusWorldNormal.hs> // GetWorldNormal
	#include <prometheusVertexDiffuse.hs> // GetVertexDiffuse
	#include <prometheusLightingUniforms.hs>

	#define AddDynamicDiffuse AddDynamicDiffuse

	void AddDynamicDiffuse()
	{
	    // directional light stuffs, directionalLight.a scales with light direction magnitude
	    // it is the scene parameter managers job to have normalized this ahead of time
	    directionalLight.a = dot(GetWorldNormal(), directionalLightDirection);
	    directionalLight.rgb = directionalLightColor * vertexColor.rgb;

	    // ambient light stuffs
	    GetVertexDiffuse().rgb = ambientColor * AmbientLightIntensity;

	   	#ifdef SetShadowmapCoord
	    	SetShadowmapCoord();
	    #endif
	}

#endif

#ifdef Pixel

	#ifdef GetIsInShadow
		#define GetDirectionalLight GetDirectionalLight

		Vec3_m GetDirectionalLight()
		{
			Float_m shadowAffected = GetIsInShadow()
			? min(0.0, directionalLight.a)
			: directionalLight.a;

			return directionalLight.rgb * max(0.0, shadowAffected * 0.8 + 0.2);
		}
	#else
		#define GetDirectionalLight() directionalLight.rgb * max(0.0, directionalLight.a * 0.8 + 0.2)
	#endif
#endif