
//
//  dualScrolling.hs
//
//  Created by Joseph. (joseph@z2.com)
//



#define FullEffect defined(texture2_available) && defined(texture3_available)



#ifdef Vertex

	varying Vec2_h texture1_uv;
	uniform Float_h gUScrollSpeed1;
	uniform Float_h gVScrollSpeed1;

	#if FullEffect

		varying Vec2_h texture2_uv;
		varying Vec2_h texture3_uv;
		uniform Float_h gUScrollSpeed2;
		uniform Float_h gVScrollSpeed2;
		uniform Float_h gUScrollSpeed3;
		uniform Float_h gVScrollSpeed3;

		void ModifyTexture1UV()
		{
			texture1_uv = vertexUV0 + fract(time * Vec2(gUScrollSpeed1, gVScrollSpeed1));
			texture2_uv = vertexUV1 + fract(time * Vec2(gUScrollSpeed2, gVScrollSpeed2));
			texture3_uv	= vertexUV0 + fract(time * Vec2(gUScrollSpeed3, gVScrollSpeed3));
		}

	#else

		void ModifyTexture1UV()
		{
			texture1_uv = vertexUV0 + fract(time * Vec2(gUScrollSpeed1, gVScrollSpeed1));
		}

	#endif

	#define ModifyTexture1UV ModifyTexture1UV

#endif

#ifdef Pixel

	#include <prometheusVertexTransparency.hs> // GetVertexTransparency

	varying Vec2_h texture1_uv;

	#if FullEffect

		varying Vec2_h texture2_uv;
		varying Vec2_h texture3_uv;
		uniform Float_m gOpacityFactor1;
		uniform Float_m gOpacityFactor2;
		uniform Float_h gDistortionIntensity;

		void overlay()
		{
			Vec4_l offset = FETCH_TEXEL_2D(texture3, texture3_uv);
			Vec4_l layer1 = FETCH_TEXEL_2D(texture1, texture1_uv + (2.0 * (offset.rg * gDistortionIntensity) - 1.0) );
			Vec4_l layer2 = FETCH_TEXEL_2D(texture2, texture2_uv + (2.0 * (offset.rg * gDistortionIntensity) - 1.0) );

			layer1.a *= gOpacityFactor1;
			layer2.a *= gOpacityFactor2;

		    outputColor = mix(layer2, layer1, layer1.a);
		    outputColor.a *= GetVertexTransparency();
		}	

	#else

		void overlay()
		{
			outputColor = FETCH_TEXEL_2D(texture1, texture1_uv);
			outputColor.a *= 0.5;
		}

	#endif

	#define ApplySurfaceColor() overlay()

#endif


