varying Vec4_h color;
varying Vec4_h worldPosition;
varying Vec4_h worldNormal;

//------------------------------------------------------------------------------
#ifdef Vertex

uniform Mat4_h worldMatrix;

void main()
{
    color = colorUniform;

    worldPosition =
        MULT_MAT_VEC(
            worldMatrix,
            Vec4(vertexPosition.xyz, 1.0));

    worldNormal =
        normalize(
            MULT_MAT_VEC(
                worldMatrix,
                Vec4(vertexNormal.xyz, 0.0)));

    pixelPosition = MULT_MAT_VEC(viewProjMatrix, worldPosition);
}

#endif // Vertex
//------------------------------------------------------------------------------
#ifdef Pixel

Vec4_h black = Vec4(0.0, 0.0, 0.0, 1.0);
Vec4_h white = Vec4(1.0, 1.0, 1.0, 1.0);

void main()
{
    Float_h alpha = 0.0;

    Vec3_h grid_interval = Vec3(1.0);

    Vec3_h p = worldPosition.xyz / grid_interval;

    // distance from camera
    Float_h d       = length(p - eye);
    Float_h alpha_d = 1.0 - smoothstep(0.0, 50.0, d);

    Float_h up = worldNormal.y;

    Vec3_h one = Vec3(1.0);
    Vec3_h two = Vec3(2.0);

    // distance from grid, [0..0.5]
    Vec3_h from_grid = max(fract(p), fract(one - p));

    // nearness to grid [0..1]
    Vec3_h near_grid = one - (from_grid * 2.0);

    Vec3_h pow_near_grid = pow(near_grid, Vec3(20.0));

    Float_h on_grid_edge = max(pow_near_grid.x, pow_near_grid.z);

    // tile alignment
    Vec3_h floor_p = floor(p);

    Vec3_h floor_p_mod2 = mod(floor_p, two);

    Float_h in_odd_tile = abs(floor_p_mod2.x - floor_p_mod2.z);

    alpha
        = (on_grid_edge + in_odd_tile * 0.15)
        * alpha_d
        * pow(up, 50.0)
        * 0.4;

    outputColor = Vec4(black.rgb, alpha);
}

#endif // Pixel