varying Vec4_h color;
varying Vec4_h worldPosition;
varying Vec4_h worldNormal;
varying Vec4_h worldCenter;
varying Vec4_h worldAxis;

//------------------------------------------------------------------------------
#ifdef Vertex

uniform Mat4_h worldMatrix;

void main()
{
    color = colorUniform;

    worldCenter = 
        MULT_MAT_VEC(
            worldMatrix,
            Vec4(0.0, 0.0, 0.0, 1.0));

    worldAxis =
        normalize(
            MULT_MAT_VEC(
                worldMatrix,
                Vec4(0.0, 1.0, 0.0, 0.0)));

    worldPosition =
        MULT_MAT_VEC(
            worldMatrix,
            Vec4(vertexPosition.xyz, 1.0));

    worldNormal =
        normalize(
            MULT_MAT_VEC(
                worldMatrix,
                Vec4(vertexNormal.xyz, 0.0)));

    pixelPosition = MULT_MAT_VEC(viewProjMatrix, worldPosition);
}

#endif // Vertex
//------------------------------------------------------------------------------
#ifdef Pixel

Vec4_h black = Vec4(0.0, 0.0, 0.0, 1.0);
Vec4_h white = Vec4(1.0, 1.0, 1.0, 1.0);

struct Sphere {
    Vec3_h  center;
    Float_h radius;
};

// absolute distance from point to sphere
Float_h distance(Vec3_h p, Sphere s)
{
    Float_h d_center = length(p - s.center);

    return abs(d_center - s.radius);
}

void main()
{
    Float_h R = 1.0;

    Sphere sphere;
    sphere.center = worldCenter.xyz;
    sphere.radius = R;

    Float_h d_sphere = distance(worldPosition.xyz, sphere);

    Float_h d_plane = abs(worldPosition.y - worldCenter.y);

    Vec3_h UP = Vec3(0.0, 1.0, 0.0);

    Float_h horizontal = dot(worldNormal.xyz, worldAxis.xyz);
    Float_h vertical = 1.0 - horizontal;

    Float_h d_ring = (d_sphere * vertical) + (d_plane * vertical);

    Float_h near_ring = 1.0 - d_ring;

    Float_h LINE_WIDTH = 0.01;

    Float_h d_eye = length(worldPosition.xyz - eye);

    Float_h HALF_WIDTH = LINE_WIDTH / 2.0 * d_eye;

    Float_h R0 = R - HALF_WIDTH;
    Float_h R1 = R + HALF_WIDTH;

    Float_h alpha = 
        smoothstep(R0, R, near_ring) -
        smoothstep(R, R1, near_ring);

    outputColor = Vec4(black.rgb, pow(alpha, 3.0));
}

#endif // Pixel