//
//  lambert_driveL.vs
//
//  Created by Joseph Danaher. (joseph@z2.com)
//  Copyright (c) 2013 Z2Live. All rights reserved.
//
//  Identical to the lambert vertex shader, with the exception
//	that the light conribution is driven from the
//	vertex alpha.
//

#include <DepthFogging.hs>
#include <DirectionalLighting.hs>
#include <WorldSpaceNormal.hs>
#include <ModelTinting.hs>

// uniforms
uniform Mat4_h worldMatrix;
uniform Vec3_m directionalLightDirection;
uniform Vec3_m directionalLightColor;
uniform Vec4_h vertColorScalingUniform;
uniform Vec3_m ambientColor;
uniform Float_h AmbientLightIntensity;

// varyings
varying Vec4_l pixelColorScale;
varying Vec3_m pixelNormal;
varying Vec4_h debugColor;

void main(){

	debugColor.xyz = vertexNormal;



    pixelPosition = MULT_MAT_VEC(viewProjMatrix, MULT_MAT_VEC(worldMatrix, vec4(vertexPosition, 1.0)));

    pixelNormal = getWorldSpaceNormal(vertexNormal);

	Vec4_h lighting = GenerateDirectionalLightColor_reduct(
						pixelNormal,
						directionalLightDirection,

						ambientColor,
						AmbientLightIntensity,

						directionalLightColor,
						vertexColor.a
	);

	pixelColorScale = vertexColor * lighting;

    ApplyTint();

    GeneratePixelFog();
}
