//
//  lightmap.vs
//
//  Created by Joseph Danaher (Joseph@z2.com)
//  Copyright (c) 2013 Z2Live. All rights reserved.
//

#include <DepthFogging.hs>
#include <DirectionalLighting.hs>
#include <WorldSpaceNormal.hs>
#include <ModelTinting.hs>

// uniforms
uniform Mat4_h worldMatrix;
uniform Vec3_m directionalLightColor;
uniform Vec3_m directionalLightDirection;
Float_h LightIntensity = 1.0;
uniform Vec4_h vertColorScalingUniform;
uniform Vec3_m ambientColor;
uniform Float_h AmbientLightIntensity;


uniform Float_h gUScrollSpeed;
uniform Float_h gVScrollSpeed;


// varyings
varying Vec4_l pixelColorScale;
varying Vec3_h pixelNormal;
varying Vec4_h debugColor;


void main(){

    debugColor.xyz = vertexNormal;



    pixelPosition = MULT_MAT_VEC(viewProjMatrix, MULT_MAT_VEC(worldMatrix, Vec4(vertexPosition.xyz, 1.0)));

    pixelNormal = getWorldSpaceNormal(vertexNormal);

    Vec4_h lighting = GenerateDirectionalLightColor(
                        pixelNormal,
                        directionalLightDirection,

                        ambientColor,
                        AmbientLightIntensity,

                        directionalLightColor,
                        LightIntensity
    );

    pixelColorScale = vertexColor * lighting;

    texture1_uv += fract(time * Vec2(gUScrollSpeed, gVScrollSpeed));

    ApplyTint();

    GeneratePixelFog();
}
