//
//  phong.vs
//
//  Created by Pingu. (joseph@z2.com)
//  Copyright (c) 2014 Z2Live. All rights reserved.
//

#include <SpecularHilighting.hs>
#include <DirectionalLighting.hs>
#include <DepthFogging.hs>
#include <WorldSpaceNormal.hs>
#include <ModelTinting.hs>
#include <Skinning.hs>
#include <SkinnedSpaceNormals.hs>

// uniforms
uniform Mat4_h  worldMatrix;
uniform Vec3_m  directionalLightColor;
uniform Vec3_m  directionalLightDirection;
uniform Vec3_m  SpecularLightDirection;
Float_h LightIntensity = 1.0;
uniform Vec4_h  vertColorScalingUniform;
uniform Vec3_m  ambientColor;
uniform Float_h AmbientLightIntensity;



// varyings
varying Vec4_l pixelColorScale;
varying Vec3_m pixelNormal;
varying Vec4_h debugColor;

varying Vec4_h debug;
varying Vec3_m pixelViewDir;
varying Vec3_m pixelSpecular;

void main(){

    // Add skinning
    Mat4_h skinnedWorldMatrix = worldMatrix * SkinnedTransform(vertexBoneWeights, vertexBoneIndices);

    Vec3_h worldPos = MULT_MAT_VEC(skinnedWorldMatrix, Vec4(vertexPosition.xyz, 1.0)).xyz;

    pixelViewDir = normalize(worldPos - eye);

    pixelNormal = getSkinnedSpaceNormal( vertexNormal, skinnedWorldMatrix );

    pixelSpecular = reflect(SpecularLightDirection, pixelNormal);

    pixelPosition = MULT_MAT_VEC(viewProjMatrix, MULT_MAT_VEC(skinnedWorldMatrix, Vec4(vertexPosition.xyz, 1.0)));

    Vec4_h lighting = GenerateDirectionalLightColor(
                        pixelNormal,
                        directionalLightDirection,

                        ambientColor,
                        AmbientLightIntensity,

                        directionalLightColor,
                        LightIntensity
    );

    pixelColorScale = vertexColor * lighting;

    ApplyTint();

    GeneratePixelFog();

}

