
//
//  prometheusSkinned.hs
//
//  Created by Aaron Bossert. (aaronb@z2.com)
//

// Provide functions:
// - InitWorldTransform(), to be called from shader main
// - ToWorldSpace(), to be called from shader stages

// Anticipations:
// 1.) to be included by multiple files in the same shader main, so introduction of new variables and macros is guarded
// 2.) mutiple calls to ToWorldSpace in the same shader, so worldTransform is stored in a var
// 3.) some platforms may struggle with vertex throughput, if so use one bone per vert


#ifndef ToWorldSpace

	#define WorldYTranslation() (worldMatrix[3][1])

	#define WorldTranslation() (Vec3(worldMatrix[3][0], worldMatrix[3][1], worldMatrix[3][2]))

	#define ToWorldSpace(objectSpace) MULT_MAT_VEC(worldTransform, objectSpace)

	#define InitWorldTransform() Mat4_h worldTransform = GenerateWorldTransform()

	#ifdef Vertex

		#define MAX_BONES 25

		uniform Mat4_h worldMatrix;
		uniform Mat4_h bones[MAX_BONES];

		Mat4_h GenerateWorldTransform()
		{
			#if PERFORMANCE_VERT > PERFORMANCE_LOW

			    IVec4 index = IVec4(clamp(vertexBoneIndices, 0.0, Float(MAX_BONES) - 1.0));

			    return worldMatrix *
				    ( vertexBoneWeights.x * bones[index.x]
				    + vertexBoneWeights.y * bones[index.y]
				    + vertexBoneWeights.z * bones[index.z]
				    + vertexBoneWeights.w * bones[index.w] );

			#else

				int index = int(clamp(vertexBoneIndices.r, 0.0, Float(MAX_BONES) - 1.0));

			    return worldMatrix * bones[index];

			#endif
		}

	#endif

#endif





