
//
//  prometheusWorldNormal.hs
//
//  Created by Aaron Bossert. (aaronb@z2.com)
//

// Provide functions:
// - InitWorldNormal(), to be called from shader main
// - GetWorldNormal(), to be called from shader stages

// Anticipations:
// 1.) to be included in multiple files included in the same shader main
// 1.) if the current platform has issues with fill, we should use per vertex normals in the pixel shader

#ifndef GetWorldNormal

varying Vec3_h worldNormalVarying;

#ifdef Vertex

	#define InitWorldNormal() worldNormalVarying = normalize(ToWorldSpace(Vec4(vertexNormal.xyz, 0.0)).xyz);

	#define GetWorldNormal() worldNormalVarying

#endif


#ifdef Pixel

	#if PERFORMANCE_FILL > PERFORMANCE_LOW

		Vec3_h perPixelNormal = normalize(worldNormalVarying);

		#define GetWorldNormal() perPixelNormal

	#else

		#define GetWorldNormal() worldNormalVarying

	#endif

#endif


#endif