

#include <prometheusVertexTransparency.hs> // GetVertexTransparency

uniform Float_h WindSpeed;  // Speed at which the body bends
uniform Float_h gBendAmount;    // How much the body bends
uniform Vec3_h WindDirection;


// Triangle wave floats
Float_h triangleWave( Float_h x ) {
  return abs( fract( x + 0.5 ) * 2.0 - 1.0 );
}
Float_h smoothCurve( Float_h x ) {
  return x * x * ( 3.0 - 2.0 * x );
}
Float_h smoothTriangleWave( Float_h x ) {
  return smoothCurve( triangleWave( x ) );
}

Float_h calculateBendFactor( Float_h bendScale, Float_h vertex ){

    //Float_h bendScale = 0.06 * ( smoothTriangleWave( (time + ( worldTranslation.x + worldTranslation.z )) * 0.4 ));

    // Bend Factor
    Float_h bendFactor = vertex * bendScale;
    bendFactor += 1.0;
    bendFactor *= bendFactor;
    bendFactor = bendFactor * bendFactor - bendFactor;
    return bendFactor;
}

Vec4_h wind( Vec4_h position, Vec3_h worldTranslation, Vec3_h worldMeshOrigin ){

    //Vec3_h worldTranslation = WorldTranslation();
    // World Position is the translation of the vertex in the world space
    Vec3_h worldPosition = position.xyz - worldTranslation - worldMeshOrigin;
    //Vec3_h vWind = Vec3(1.0, 0.0, 1.0); //debug

    // Get the distance of the vert from 0
    // We want to make sure the object bends, not shears
    Float_h dist = length( worldPosition );

    // Bend Scale
    // gBendTimeCoef
    // gBendCoef
    // DEBUG MODE __ Should use the uniforms
    //Float_h bendFactor1 = calculateBendFactor( (gBendAmount+0.15) * ( smoothTriangleWave( (time) * WindSpeed ) ), worldPosition.y );
    //Float_H bendFactor2 = 0.0;
    Float_h bendFactor1 = calculateBendFactor( gBendAmount * ( smoothTriangleWave( (time + ( worldTranslation.x + worldTranslation.z + worldMeshOrigin.x ) ) * WindSpeed ) ), worldPosition.y );
    Float_h bendFactor2 = calculateBendFactor( gBendAmount * 0.5 * ( smoothTriangleWave( (time + ( worldTranslation.z * worldTranslation.x + worldMeshOrigin.z ) ) * WindSpeed * 0.5 ) ), worldPosition.y );

    // Move the verts in the wind direction using bend factor
    worldPosition.xz += WindDirection.xz * bendFactor1;

    //worldPosition.xz += Vec2_h(WindDirection.z, -WindDirection.x) * sin(gBendTimeCoef);//hahah
    worldPosition.xz += Vec2( WindDirection.z, -WindDirection.x ) * bendFactor2;

    // Apply the original length back to the vert
    worldPosition.xyz = normalize( worldPosition.xyz ) * dist;// + ( ( (worldPosition.x - x) * 0.2) * GetVertexTransparency() );

    //worldPosition = ApplyDetailBending(worldPosition, vertexPosition, vertexNormal, vertexUV1); // This should be an option, but I've disabled it for now.

    return Vec4(worldPosition + worldTranslation + worldMeshOrigin, 1.0);

}
